@@ -44,6 +44,7 @@
 #else
 
 # include <unistd.h>
+# include <strings.h>
 # ifdef GIT_THREADS
 #	include <pthread.h>
 #	include <sched.h>
@@ -57,7 +58,6 @@
 #include "git2/types.h"
 #include "git2/errors.h"
 #include "thread-utils.h"
-#include "bswap.h"
 
 #include <regex.h>
 
@@ -67,7 +67,7 @@
 #define GITERR_CHECK_ALLOC(ptr) if (ptr == NULL) { return -1; }
 
 /**
- * Check a return value and propogate result if non-zero.
+ * Check a return value and propagate result if non-zero.
  */
 #define GITERR_CHECK_ERROR(code) \
 	do { int _err = (code); if (_err) return _err; } while (0)
@@ -169,6 +169,11 @@ GIT_INLINE(void) git__init_structure(void *structure, size_t len, unsigned int v
 }
 #define GIT_INIT_STRUCTURE(S,V) git__init_structure(S, sizeof(*S), V)
 
+#define GIT_INIT_STRUCTURE_FROM_TEMPLATE(PTR,VERSION,TYPE,TPL) do { \
+	TYPE _tmpl = TPL; \
+	GITERR_CHECK_VERSION(&(VERSION), _tmpl.version, #TYPE);	\
+	memcpy((PTR), &_tmpl, sizeof(_tmpl)); } while (0)
+
 /* NOTE: other giterr functions are in the public errors.h header file */
 
 #include "util.h"
