@@ -1,5 +1,5 @@
 package Git::Raw;
-$Git::Raw::VERSION = '0.50';
+$Git::Raw::VERSION = '0.51';
 use strict;
 use warnings;
 
@@ -16,7 +16,10 @@ use Git::Raw::Tree;
 
 =for HTML
 <a href="https://travis-ci.org/jacquesg/p5-Git-Raw">
-	<img src="https://travis-ci.org/jacquesg/p5-Git-Raw.svg?branch=master" alt="Build Status" align="right" />
+	<img src="https://travis-ci.org/jacquesg/p5-Git-Raw.svg?branch=master" alt="Build Status: Travis" align="right" />
+</a>
+<a href="https://ci.appveyor.com/project/jacquesg/p5-git-raw">
+	<img src="https://ci.appveyor.com/api/projects/status/il9rm9fsf9dj1dcu/branch/master?svg=true" alt="Build Status: AppVeyor" align="right" />
 </a>
 <a href="https://coveralls.io/r/jacquesg/p5-Git-Raw">
 	<img src="https://coveralls.io/repos/jacquesg/p5-Git-Raw/badge.png?branch=master" alt="Coverage Status" align="right" />
@@ -29,7 +32,7 @@ Git::Raw - Perl bindings to the Git linkable library (libgit2)
 
 =head1 VERSION
 
-version 0.50
+version 0.51
 
 =head1 DESCRIPTION
 
